@@ -14,7 +14,7 @@ use overload
 
 use if ($] >= 5.010), overload => '~~'  => "matches";
 
-our $VERSION = '2.01';
+our $VERSION = '2.02';
 
 my $PACKAGE = __PACKAGE__;  # Useful to have a scalar for hash keys.
 
@@ -127,6 +127,31 @@ The line in C<< $E->file >> where the exceptional code was called.
 
 sub line        { return $_[0]->{$PACKAGE}{line};  }
 
+=head3 context
+
+    my $context = $E->context;
+
+The context in which the subroutine was called.  This can be
+'list', 'scalar', or undefined (unknown).  It will never be 'void', as
+C<autodie> always captures the return value in one way or another.
+
+=cut
+
+sub context     { return $_[0]->{$PACKAGE}{context} }
+
+=head3 return
+
+    my $return_value = $E->return;
+
+The value(s) returned by the failed subroutine.  When the subroutine
+was called in a list context, this will always be a reference to an
+array containing the results.  When the subroutine was called in
+a scalar context, this will be the actual scalar returned.
+
+=cut
+
+sub return      { return $_[0]->{$PACKAGE}{return} }
+
 =head3 errno
 
     my $errno = $E->errno;
@@ -558,6 +583,8 @@ sub format_default {
         args => \@_,
         function => "CORE::open",
         errno => $!,
+        context => 'scalar',
+        return => undef,
     );
 
 
@@ -658,6 +685,9 @@ sub _init {
 
     $this->{$PACKAGE}{errno}   = $args{errno} || 0;
 
+    $this->{$PACKAGE}{context} = $args{context};
+    $this->{$PACKAGE}{return}  = $args{return};
+
     $this->{$PACKAGE}{args}    = $args{args} || [];
     $this->{$PACKAGE}{function}= $args{function} or
               croak("$class->new() called without function arg");
